
!---------------------------------------
! Simon's coincidence loss test routine
! 27.05.2004
!---------------------------------------
subroutine coincLossAsymErr(frameTime, timeBin, trueCountRate, &
     lossCountRate, dispersionLeft, dispersionRight, rmsValue, verbFlag)
  real, intent(in) :: frameTime
  real, intent(in) :: timeBin
  real, intent(in) :: trueCountRate ! counts per second
  real, intent(out) :: lossCountRate
  real, intent(out) :: dispersionLeft
  real, intent(out) :: dispersionRight
  real, intent(out) :: rmsValue
  integer, intent(in) :: verbFlag
  integer :: nFrames

!*****************************************************************
  character(len=50) fout

  real(kind=double) :: pi, prob_m_k_mu, mu, ln_p, p
  real(kind=double) :: ln_k_fact, ln_m_fact, ln_k_m_fact, sum_p
  real(kind=double) :: p_poiss, ln_p_poiss, sum_p_poiss, mu_poiss
  real(kind=double) :: rms, mean, r2, errl, errh, devl, devh, cl_factor
  real(kind=double) :: cl_formula, mean_poiss, sigma_poiss, sum_p_l, sum_p_h

  integer(kind=int32) :: k, m, i, max

  real(kind=single) :: ci, t ! incident CR (cts/s) and frame rate (s)
  real(kind=single) :: ci_low, ci_hi, co_low, co_hi

  pi=4.d0*atan(1.d0)

  nFrames=int(timeBin/frameTime)
!  write(*,*) " ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^"
!  write(*,*) " L2499 coincLossTest frameTime=", frameTime, " timeBin=", timeBin, &
!       " nFrames=", nFrames, " trueCountRate=", trueCountRate

!      ci -> trueCountRate
!      t -> frameTime
  ci=trueCountRate
  t=frameTime

  mu_poiss=ci ! incident counts during one second
!  write(*,*) " L2515 mu_poiss=", mu_poiss, " (incident counts during one second)"
  mu=ci*t ! incident counts during one frame
!  write(*,*) " mu=", mu, " (incident counts per one frame)"
  cl_formula=(1.d0-exp(-mu))/frameTime !t
!  write(*,*) " L2519 cl_formula=", cl_formula, " (per second)"
! can prompt for no of frames but for 1 second of data, use k=int(1/t)
!
!      write(*,'('' How many frames (k must be > = m)? -> '',$)')
!      read(*,*)k
  k=nFrames ! total number of frames per exposure time (timeBin)
!  write(*,*) " L2525 Total number of frames (per timeBin) k=", k

! First compute the probability distribution of the incident count rate, for 
! reference
!
  sum_p_poiss=0.d0
  mean_poiss=0.d0
  sigma_poiss=0.d0

  max=int(5*Ci)
  if (ci.lt.10) max=30

!  write(*,*) " L2541 Ci=", Ci, "max=int(5*Ci)=", max

  do i=0,max
     m=i
     ln_m_fact=calc_ln_fact(m)
     ln_p_poiss = m*log(mu_poiss) - mu_poiss - ln_m_fact
     p_poiss = exp(ln_p_poiss)
     if(p_poiss.lt.1.d-30)p_poiss=0.d0
     sum_p_poiss = sum_p_poiss + p_poiss
     mean_poiss=mean_poiss+p_poiss*m
         
  end do




  if (verbFlag .eq. 1) write(*,*) " L2556 mean_poiss=", mean_poiss
!
! As a check, compute sigma using sqrt[sum(p*(xi-X)**2] rather than 
! sqrt[bar(r**2) - (bar(r))**2]
!
  devl=0.d0
  devh=0.d0
  sum_p_l=0.d0
  sum_p_h=0.d0

  do i=0,max
     m=i
     ln_m_fact=calc_ln_fact(m)
     ln_p_poiss = m*log(mu_poiss) - mu_poiss - ln_m_fact
     p_poiss = exp(ln_p_poiss)
     if(p_poiss.lt.1.d-30)p_poiss=0.d0
     sigma_poiss=sigma_poiss + p_poiss * (1.d0*m-mean_poiss)**2
     if(m.le.mean_poiss)then
        devl=devl + p_poiss*(m-mean_poiss)**2
        sum_p_l=sum_p_l+p_poiss
     end if
     if(m.gt.mean_poiss)then
        devh=devh + p_poiss*(m-mean_poiss)**2
        sum_p_h=sum_p_h+p_poiss
     end if
  end do

!      write(1,'(//)')

  if (verbFlag .eq. 1) then
     write(*,*) " For input Poisson distribution "
     write(*,*) " ------------------------------ "
     write(*,*) " L2587 mean_poiss=", mean_poiss, " sqrt(sigma_poiss)=", &
          sqrt(sigma_poiss)
     write(*,*) " Sum of P(n,mu)=", sum_p_poiss, " (should be 1.0)"
     write(*,*) " devl=", devl, " devh=", devh, " sum_p_l=", sum_p_l, &
          " sum_p_h=", sum_p_h
     write(*,*) " sqrt(devl/sum_p_l)=", sqrt(devl/sum_p_l), &
          " sqrt(devh/sum_p_h)=", sqrt(devh/sum_p_h)
  endif

!  compute the probability of getting m=i counts if we run k frames with an 
!  incident rate of Ci counts/s and a frame time of t seconds. This 
!  probability is based on kCm * P(0)**(k-m) * P(>=1)**m 
!
!  where
!
!  kCm = k!/(m!)(k-m)!
!
!
  sum_p=0.d0
  rms=0.d0
  r2=0.d0
  mean=0.d0
  ln_k_fact=calc_ln_fact(k)        ! constant for given no of frames

!      do i=0,5*k
!  write(*,*)'***********************'
  do i=0,k  ! m can not exceed k
     m=i        
     ln_m_fact=calc_ln_fact(m)
     ln_k_m_fact=calc_ln_fact(k-m)
     ln_p = ln_k_fact - ln_m_fact - ln_k_m_fact
     ln_p = ln_p - (k-m)*mu + m*log(1.d0-exp(-mu))
     p=exp(ln_p)
     if(p.lt.1.d-30)p=0.d0    ! set to zero if P is very small. 
     mean=mean+p*m
     r2=r2 + p*m*m
!        write(1,*)' k,m,k-m,ln_k_fact,ln_m_fact,ln_k_m_fact,&
!         &ln_p,p,sum_p ='&
!         &,k,m,k-m,ln_k_fact,&
!         &ln_m_fact,ln_k_m_fact,ln_p,p, sum_p

     sum_p=sum_p+p
!     write(*,*) " L2633 m=",m, " p=", p, " sum_p=", sum_p
!        write(1,*)1.0*m,p
  end do

  rms=r2-mean*mean    ! rms (square of)

!  write(*,'(" For the coincidence loss affected (truncated) &
!       &distribution",/)')
!  write(*,*) " sum_p=", sum_p, " (should be 1.0)"
!  write(*,*) " Mean of observed counts (during timeBin) mean=", mean, &
!       " cl_formula*k*t=", cl_formula*k*frameTime ! t

  if(rms.ge.0.d0)then
     rms=sqrt(rms)
!     write(*,'("RMS = ",f10.5)')rms
  else
     if (verbFlag .eq. 1) write(*,*)'r2-mean*mean < 0 !'
  end if
  !cl_factor=ci*t*k/mean
  cl_factor=ci*frameTime*k/mean
!  write(*,'("Coincidence loss factor (Ci/mean) = ",f10.5)')cl_factor

!
! now compute asymmetric errors - For asymmetric errors need to know 
! what mean is first, so must compute above loop first and then repeat to 
! get deviations from mean, weighted by P
!

  errl=0.d0
  errh=0.d0
  devl=0.d0
  devh=0.d0
  rms=0.d0

!      do i=0,5*k
  do i=0,k     ! m can not exceed k
     m=i
     ln_m_fact=calc_ln_fact(m)
     ln_k_m_fact=calc_ln_fact(k-m)
     ln_p = ln_k_fact - ln_m_fact - ln_k_m_fact
     ln_p = ln_p - (k-m)*mu + m*log(1.d0-exp(-mu))
     p=exp(ln_p)
     if (p.lt.1.d-30) p=0.d0
     rms=rms+p*(m-mean)**2
     if (m.le.mean) devl=devl + p*(m-mean)**2
     if (m.gt.mean) devh=devh + p*(m-mean)**2
!        if(m.le.mean)write(1,*)' m,p,mean,devl = ',m,p,mean,devl
!        if(m.gt.mean)write(1,*)' m,p,mean,devh = ',m,p,mean,devh
  end do
!  write(*,*) " L2682 m=",m," p=",p, " mean=", mean

  errl=sqrt(devl)
  errh=sqrt(devh)
  if (verbFlag .eq. 1) write(*,*) "L2765  devl=", devl, "errl=", errl, " devh=", devh, " errh=", errh, " rms=", rms
  if (rms .gt. 0.d0) then
     rms=sqrt(rms)
  else
     if (verbFlag .eq. 1) write(*,*) " L2771 rms is megative"
  endif
!  dispersionLeft=devl/timeBin
!  dispersionRight=devh/timeBin
!  lossCountRate=mean/timeBin
!
  rmsValue=rms/timeBin
  dispersionLeft=errh/timeBin
  dispersionRight=errh/timeBin
  lossCountRate=mean/timeBin
!!      co_low=mean-errl
!!      co_hi=mean+errh

!!      ci_low=-log(1.d0-co_low*t)/t
!!      ci_hi=-log(1.d0-co_hi*t)/t
!!      ci=-log(1.d0-mean*t)/t

      

!      write(*,*)'Left err = ',errl,' Right err = ',errh
!  write(*,'("Mean  = ",f15.8,"  -",f13.8,"  +",f13.8)')mean,errl,errh

!!      write(*,'("For Incident rate, Mean  = ",f15.8,"  -",f13.8,&
!!       &"  +",f13.8)')ci,ci-ci_low, ci_hi-ci
      
  !    close(1)
end subroutine coincLossAsymErr
