!
! 
module countrateerror_mod

  use dal
  use cal
  use param  
  use errorHandling
  use frameTime_mod
  use omconstants_mod
  use omallocation_mod
  use loggamma_mod
  use sasdal

  implicit none 
  
  public rmsCountRateError




 
integer(int32) :: maxFac = 1000000 
real(double), pointer :: lFactorial(:) => null() 
 


contains 
                   
!---------------------------------------
! Simon's coincidence loss test routine
! 27.05.2004
!---------------------------------------
function rmsCountRateError(frameTime, exposure, countRate) result(rmsValue)
  real(single), intent(in) :: frameTime ! seconds
  real(single), intent(in) :: exposure  ! seconds
  real(single), intent(in) :: countRate ! counts per second
  real(single) :: rmsValue

!*****************************************************************

  real(double) :: prob_m_k_mu, mu, ln_p, p, logoneminusexpminusmu
  real(double) :: ln_k_fact, ln_m_fact, ln_k_m_fact, sum_p
  real(double) :: p_poiss, ln_p_poiss, sum_p_poiss, mu_poiss, logmu_poiss
  real(double) :: rms, mean, r2, errl, errh, cl_factor
  real(double) :: cl_formula

  integer(int32) :: k, m, i, max1

  real(single) :: ci, t ! incident CR (cts/s) and frame rate (s)
  real(single) :: ci_low, ci_hi, co_low, co_hi
 
  integer nFrames


  nFrames = int(exposure / frameTime)


!      ci -> trueCountRate
!      t -> frameTime
  ci = countRate
  t = frameTime

  mu_poiss = ci ! incident counts during one second
  logmu_poiss = log(ci)
  mu = ci * t ! incident counts during one frame
  logoneminusexpminusmu = log(1.0_double - exp(-mu))

    sum_p_poiss=0.d0
  
    

    max1 = int(5 * Ci)


    if (ci .lt. 10) max1 = 30


    rms=0.d0
    r2=0.d0
    mean=0.d0
    k = nFrames
    ln_k_fact= logFactorial(nFrames)        ! constant for given no of frames



    do i = 0, nFrames  ! m can not exceed k
            
        ln_m_fact= logFactorial(i)
        ln_k_m_fact= logFactorial(k-i)
        ln_p = ln_k_fact - ln_m_fact - ln_k_m_fact
        ln_p = ln_p - (k - i) * mu + i * logoneminusexpminusmu 
        p = exp(ln_p)
        p = max(p, 0.0d0)    ! set to zero if P is very small. 
        mean = mean + p * i
        r2 = r2 + p * i * i

    end do

    rms = r2 - mean * mean    ! rms (square of)



    if (rms .gt. 0.d0) then
        rms=sqrt(rms)
    end if

    rmsValue = rms / exposure
  

end function rmsCountRateError

!-------------------------------------
! log of factorial
!-------------------------------------
subroutine InitialiseFactorial(maxSize)
    integer(int32), intent(in) :: maxSize
    real(double), pointer :: factorial(:) => null()
    integer(int32) :: j

    maxFac = max(100, maxSize)
    lFactorial => allocateDouble(0, maxFac, "logFactorial")
    lFactorial = 0.0_double
    factorial => allocateDouble(0, maxFac, "factorial")
    factorial = 1.0_double

    do j = 1, 50
        factorial(j) = factorial(j - 1) * dble(j)
        lFactorial(j) = log(factorial(j))

    end do
    do j = 51, maxFac
        lFactorial(j) = 0.5d0 * log(TWO_PI * dble(j)) + dble(j) *(log(dble(j)) - 1.d0)
    end do

    deallocate(factorial)
    nullify(factorial)




end subroutine InitialiseFactorial

real(double) function logFactorial(i)      
    integer(int32) :: i
    real(double) :: dbleI
 
logFactorial =  Factln(i)
return
    if (i .le. maxFac) then
        logFactorial = lFactorial(i)

        return
    end if

    dbleI = dble(i)    
    logFactorial = 0.5d0 * log(TWO_PI * dbleI) + dbleI * (log(dbleI) - 1.0d0)
    call fatal("error")
end function logFactorial


real(double) function Gammln(xx)
    real(single), intent(in) :: xx
    real(double) :: x, y, tmp, ser
    real(double), save :: cof(0 : 5) = (/76.18009172947146, -86.50532032941677, &
           24.01409824083091, -1.231739572450155, 0.1208650973866179e-2, &
          -0.5395239384953e-5/)
    
    integer(int32) :: j

    x = xx; y = x
    tmp = x + 5.5_double
    tmp = tmp - (x + 0.5_double) * log(tmp)
    ser = 1.000000000190015
    do j = 0, 5
        y = y + 1.0_double
        ser = ser + cof(j) / y
    end do
    Gammln = -tmp + log(2.5066282746310005 * ser / x)
end function Gammln

real(double) Function Factln(n)
    integer(int32), intent(in) :: n
    integer(int32), parameter :: maxSize = 100000
    integer(int32) :: i
    real(double), save :: a(0 : maxSize) = (/(0.0_double, i = 0, maxSize) /)
! integer :: save, count = 0
  FactLn = calc_ln_fact(n)
return
    
    if (n .le. 1) then
        FactLn = 0.0_double
        return
    end if
    if (n .le. maxSize) then
        if (a(n) .gt. 0.0_double) then
            FactLn = a(n)
            return
         else
!            FactLn = Gammln(real(n + 1))
            FactLn = logGamma(dble(n + 1))
            a(n) = FactLn
            return
         end if
    end if
!    FactLn = Gammln(real(n + 1))
    FactLn = logGamma(dble(n + 1))
!   write(*, *) "n=", n, " FactLn=", FactLn, " original=",  calc_ln_fact(n)
!count = count + 1
!if (count .gt. 50) call fatal("stopped")      
end function FactLn
real(double) Function Bico(n, k)
    integer(int32), intent(in) :: n, k

    Bico = floor(0.5_double + exp(Factln(n) - Factln(k) - factln(n - k)))


end function Bico        









      function calc_ln_fact(i)
        
        integer(int32) i, j

        real(double) ln_i_fact, pi, calc_ln_fact, i_fact

        pi=4.d0*atan(1.d0)    !3.141592654d0

        if(i.le.50)then

           j=1
           i_fact=1.d0      ! set for 0 factorial

           do while(j.le.i)
              i_fact=i_fact*j
              j=j+1
           end do

           ln_i_fact=log(i_fact)   ! convert to a log

         else   !use Stirling's approximation


            ln_i_fact=0.5d0*log(2.d0*pi*i) + 1.d0*i*(log(1.d0*i)-1.d0)

         end if
         

      calc_ln_fact=ln_i_fact
end  function calc_ln_fact
   
    
    


   
end module countrateerror_mod








